// DynPointer by Kopie
// Modified by AubreyTheBard

#define PARAM_SCAN      // Uncomment this to activate parameter scan, which will output
                        // the unmodified parameters of event commands to a file in the
                        // DynPlugins directory called DynPointerParams.txt. This is
                        // useful for learning how the event commands work internally.

#include <DynRPG/DynRPG.h>

#ifdef PARAM_SCAN
#include <iostream>
#include <fstream>
#endif

enum var_type { VAR_RND = 3,
                VAR_ITEM = 4,
                VAR_HERO = 5,
                VAR_SPRITE = 6};

#ifdef PARAM_SCAN
std::ofstream ofParams("DynPlugins\\DynPointerParams.txt");
#endif

int random;
std::pair<int, int> item;
std::pair<int, int> hero;
std::pair<int, int> range;
std::pair<int, int> sprite;
int message_options[5];
int animation_ib[4];
int animation[5];


#ifdef PARAM_SCAN
static void recordParameters(RPG::EventScriptLine *scriptLine)
{
    static int i;       // Increment variable

    ofParams << "String parameter: " << scriptLine->stringParameter.s_str() << "\n";
    ofParams << "Integer parameters:";
    for(i = 0; i < scriptLine->parameters.size; i++)
    {
        ofParams << " " << scriptLine->parameters.items[i];
    }
    ofParams << "\n\n";
    return;
}
#endif

void onInitTitleScreen()
{
    random = 0;
    item.first = 0;
    item.second = 0;
    hero.first = 0;
    hero.second = 0;
    range.first = 0;
    range.second = 0;
    sprite.first = 0;
    sprite.second = 0;
    animation_ib[0] = 0;
    animation_ib[1] = 0;
    animation_ib[2] = 0;
    animation_ib[3] = 0;
    animation[0] = 0;
    animation[1] = 0;
    animation[2] = 0;
    animation[3] = 0;
    animation[4] = 0;
}

#ifdef PARAM_SCAN
void onExit()
{
    // Close the parameter scan output file
    ofParams.close();
}
#endif

bool onComment( const char* text,
                const RPG::ParsedCommentData* 	parsedData,
                RPG::EventScriptLine* 	nextScriptLine,
                RPG::EventScriptData* 	scriptData,
                int 	eventId,
                int 	pageId,
                int 	lineId,
                int* 	nextLineId )
{
    std::string cmd = parsedData->command;

    if(!cmd.compare("item_once"))
    {
        item.first = 1;
        item.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("item_fixed"))
    {
        item.first = -1;
        item.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("item_normal"))
    {
        item.first = 0;
        return false;
    }

    if(!cmd.compare("hero_once"))
    {
        hero.first = 1;
        hero.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("hero_fixed"))
    {
        hero.first = -1;
        hero.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("hero_normal"))
    {
        hero.first = 0;
        return false;
    }

    if(!cmd.compare("sprite_once"))
    {
        sprite.first = 1;
        sprite.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("sprite_fixed"))
    {
        sprite.first = -1;
        sprite.second = parsedData->parameters[0].number;
        return false;
    }
    if(!cmd.compare("sprite_normal"))
    {
        sprite.first = 0;
        return false;
    }

    if(!cmd.compare("random_once"))
    {
        random = 1;
        range.first = parsedData->parameters[0].number;
        range.second = parsedData->parameters[1].number;
        return false;
    }
    if(!cmd.compare("random_fixed"))
    {
        random = -1;
        range.first = parsedData->parameters[0].number;
        range.second = parsedData->parameters[1].number;
        return false;
    }
    if(!cmd.compare("random_normal"))
    {
        random = 0;
        return false;
    }
    // <added by AubreyTheBard>
    // Animation in battle comment command parameters: animation database number, target number, halting option
    if(!cmd.compare("animation_ib_once"))
    {
        animation_ib[0] = 1;
        animation_ib[1] = parsedData->parameters[0].number;
        animation_ib[2] = parsedData->parameters[1].number;
        animation_ib[3] = parsedData->parameters[2].number;
        return false;
    }
    if(!cmd.compare("animation_ib_fixed"))
    {
        animation_ib[0] = -1;
        animation_ib[1] = parsedData->parameters[0].number;
        animation_ib[2] = parsedData->parameters[1].number;
        animation_ib[3] = parsedData->parameters[2].number;
        return false;
    }
    if(!cmd.compare("animation_ib_normal"))
    {
        animation_ib[0] = 0;
        return false;
    }
    // Animation (outside battle) comment command parameters: animation database number, event target number, halting option, global scope option
    if(!cmd.compare("animation_once"))
    {
        animation[0] = 1;
        animation[1] = parsedData->parameters[0].number;
        animation[2] = parsedData->parameters[1].number;
        animation[3] = parsedData->parameters[2].number;
        animation[4] = parsedData->parameters[3].number;
        return false;
    }
    if(!cmd.compare("animation_fixed"))
    {
        animation[0] = -1;
        animation[1] = parsedData->parameters[0].number;
        animation[2] = parsedData->parameters[1].number;
        animation[3] = parsedData->parameters[2].number;
        animation[4] = parsedData->parameters[3].number;
        return false;
    }
    if(!cmd.compare("animation_normal"))
    {
        animation[0] = 0;
        return false;
    }
    return true;
    // </added by AubreyTheBard>
}

bool onEventCommand (RPG::EventScriptLine *scriptLine,
                     RPG::EventScriptData *scriptData,
                     int eventId, int pageId,
                     int lineId, int *nextLineId)
{
    #ifdef PARAM_SCAN
    if(true)       // Set to true to send parameters for this command type to DynPointerParams.txt
    {
        ofParams << "Command type: Unknown\n";
        recordParameters(scriptLine);
    }
    #endif
    if(scriptLine->command == RPG::EVCMD_CHANGE_VARIABLE)
    {
        switch(scriptLine->parameter(4))
        {
            case VAR_RND:
                if(random != 0)
                {
                    scriptLine->parameters.items[5] = range.first;
                    scriptLine->parameters.items[6] = range.second;
                    if(random > 0) random = 0;
                    return true;
                }
                break;
            case VAR_ITEM:
                if(item.first != 0)
                {
                    scriptLine->parameters.items[5] = item.second;
                    if(item.first > 0) item.first = 0;
                    return true;
                }
                break;
            case VAR_HERO:
                if(hero.first != 0)
                {
                    scriptLine->parameters.items[5] = hero.second;
                    if(hero.first > 0) hero.first = 0;
                    return true;
                }
                break;
            case VAR_SPRITE:
                if(sprite.first != 0)
                {
                    scriptLine->parameters.items[5] = sprite.second;
                    if(sprite.first > 0) sprite.first = 0;
                    return true;
                }
                break;
            default:;
        }
    }

    // <added by AubreyTheBard>

    // The Message Display Options event command has 4 integer parameters:
    // 0) Window Type
    //      0 for Normal
    //      1 for Transparent
    // 1) Window Position
    //      0 for Top
    //      1 for Middle
    //      2 for Bottom
    // 2) Autoselect Window Position to Prevent Hero from Being Obscured
    //      0 for false
    //      1 for true
    // 3) Allow Other Events to Continue While Message is Being Shown
    //      0 for false
    //      1 for true
    if(scriptLine->command == RPG::EVCMD_MESSAGE_OPTIONS)
    {
        #ifdef PARAM_SCAN
        if(false)       // Set to true to send parameters for this command type to DynPointerParams.txt
        {
            ofParams << "Command type: Message Display Options\n";
            recordParameters(scriptLine);
        }
        #endif
        if(message_options[0] != 0)
        {
            if(message_options[1] >= 0 && message_options[1] <= 1)
            {
                scriptLine->parameters.items[0] = message_options[1];             // Window Type
            }
            if(message_options[2] >= 0 && message_options[2] <= 2)
            {
                scriptLine->parameters.items[1] = message_options[2];             // Window Position
            }
            if(message_options[3] >= 0 && message_options[3] <= 1)
            {
                scriptLine->parameters.items[2] = message_options[3];             // Autoselect Window Position to Prevent Hero from Being Obscured
            }
            if(message_options[4] >= 0 && message_options[3] <= 1)
            {
                scriptLine->parameters.items[3] = message_options[4];             // Allow Other Events to Continue While Message is Being Shown
            }
            if(message_options[0] > 0)
            {
                message_options[0] -= 1;
            }
            return true;
        }
    }

    // The Face Graphics event command has 3 integer parameters:
    // 0) Face Number
    //      0 through 15, starting in upper left corner and reading left-to-right then down
    // 1) Display Position
    //      0 for false
    //      1 for true
    // 2) Flip image horizontally
    //      0 for false
    //      1 for true
    // It also has a string parameter:
    //      "" to erase the face
    //      any other string is matched to a file name minus extension in the FaceSet folder
    //
    if(scriptLine->command == RPG::EVCMD_SELECT_FACE)
    {
        #ifdef PARAM_SCAN
        if(false)       // Set to true to send parameters for this command type to DynPointerParams.txt
        {
            ofParams << "Command type: Message Display Options\n";
            recordParameters(scriptLine);
        }
        #endif
        if(message_options[0] != 0)
        {
            if(message_options[1] >= 0)
            {
                scriptLine->parameters.items[0] = message_options[1];             // Window Type
            }
            if(message_options[2] >= 0)
            {
                scriptLine->parameters.items[1] = message_options[2];             // Window Position
            }
            if(message_options[3] >= 0)
            {
                scriptLine->parameters.items[2] = message_options[3];             // Autoselect Window Position to Prevent Hero from Being Obscured
            }
            if(message_options[4] >= 0)
            {
                scriptLine->parameters.items[3] = message_options[4];             // Allow Other Events to Continue While Message is Being Shown
            }
            if(message_options[0] > 0)
            {
                message_options[0] = 0;
            }
            return true;
        }
    }

    // The Show battle animation in battle event command has 5 internal parameters:
    // 0) Battle animation database reference number, must be at least 1
    // 1) Side-agnostic animation target
    //      -1 for all of a side
    //      0-7 for monsters 1-8
    //      1, 2, 3, etc. for hero 1, hero 2, hero 3, etc.
    // 2) Halt processing until animation ends flag; 0 for no halt, 1 for halt
    // 3) Which side is targeted flag; 0 for monsters, 1 for heroes
    // 4) Side-aware animation target
    //      -1 for all monsters
    //      0-7 for monsters 1-8
    //      8 for all heroes
    //      9, 10, 11, etc. for hero 1, hero 2, hero 3, etc.
    //
    // For user simplicity, the plugin takes three arguments in the @animation_ib_X
    // comment commands:
    // 0) Battle animation database reference number (0 or less indicates no change to event command)
    // 1) Side-aware animation target (-2 or less indicates no change to event command)
    // 2) Halt processing until animation ends flag (-1 or less indicates no change to event command)
    //
    // Any necessary changes to event command parameters 1 and 3 are calculated and applied by
    // the code below based on comment command argument 1.
    if(scriptLine->command == RPG::EVCMD_SHOW_BATTLE_ANIMATION_IN_BATTLE)
    {
        #ifdef PARAM_SCAN
        if(false)       // Set to true to send parameters for this command type to DynPointerParams.txt
        {
            ofParams << "Command type: Show battle animation in battle\n";
            recordParameters(scriptLine);
        }
        #endif
        if(animation_ib[0] != 0)
        {
            if(animation_ib[1] >= 1)
            {
                scriptLine->parameters.items[0] = animation_ib[1];             // Battle animation database reference number
            }
            if(animation_ib[2] >= -1)
            {
                scriptLine->parameters.items[4] = animation_ib[2];             // Side-aware animation target
                if(animation_ib[2] == -1 || animation_ib[2] == 8)
                {
                    scriptLine->parameters.items[1] = -1;                   // Side-agnostic animation target; all of the side
                }
                else if(animation_ib[2] > 8)
                {
                    scriptLine->parameters.items[1] = animation_ib[2] - 8;     // Side-agnostic animation target; single hero
                }
                else
                {
                    scriptLine->parameters.items[1] = animation_ib[2];         // Side-agnostic animation target; single monster
                }
                if(animation_ib[2] < 8)
                {
                    scriptLine->parameters.items[3] = 0;                    // Which side is targeted flag; monsters
                }
                else
                {
                    scriptLine->parameters.items[3] = 1;                    // Which side is targeted flag; heroes
                }
            }
            if(animation_ib[3] >= 0)
            {
                scriptLine->parameters.items[3] = animation_ib[3];             // Halt processing until animation ends flag
            }
            if(animation_ib[0] > 0)
            {
                animation_ib[0] = 0;
            }
            return true;
        }
    }

    // The Show battle animation (not in battle) event command has 4 internal parameters:
    // 0) Battle animation database reference number, must be at least 1
    // 1) Animation target
    //      10001 for the hero
    //      10002 for the skiff
    //      10003 for the ship
    //      10004 for the airship
    //      10005 for "this event"
    //      1, 2, 3, etc. for event 1, event 2, event 3, etc.
    // 2) Halt Other Processes flag; 0 for no halt, 1 for halt
    // 3) Global Scope flag; 0 for local scope, 1 for global scope
    if(scriptLine->command == RPG::EVCMD_SHOW_BATTLE_ANIMATION)
    {
        #ifdef PARAM_SCAN
        if(false)       // Set to true to send parameters for this command type to DynPointerParams.txt
        {
            ofParams << "Command type: Show battle animation (outside battle)\n";
            recordParameters(scriptLine);
        }
        #endif
        if(animation[0] != 0)
        {
            if(animation[1] >= 1)
            {
                scriptLine->parameters.items[0] = animation[1];             // Battle animation database reference number
            }
            if(animation[2] >= 1)
            {
                scriptLine->parameters.items[1] = animation[2];             // Animation event target
            }
            if(animation[3] >= 0)
            {
                scriptLine->parameters.items[2] = animation[3];             // Halt processing until animation ends flag
            }
            if(animation[4] >= 0)
            {
                scriptLine->parameters.items[3] = animation[4];             // Global scope flag
            }
            if(animation[0] > 0)
            {
                animation[0] = 0;
            }
            return true;
        }
    }

    return true;
    // </added by AubreyTheBard>
}
