/*! \file main.cpp

    \brief DynRPG sandbox plugin

    \author Bernard J. Badger, AKA Aubrey the Bard
*/


#include <DynRPG/DynRPG.h>
#define NOT_MAIN_MODULE

//! Handler called on startup
/*!
    onStartup() notifies the user on starting RPG_RT.exe that the sandbox
    plugin is active and asks whether they want to proceed. This is useful
    for preventing accidental publication of a game release which includes
    the sandbox plugin.

    \return (bool) Whether the user chose to proceed
*/
bool onStartup(char *pluginName) {
               if(MessageBox(
                              NULL, // We don't need a window handle
                              "Notice: currently using sandbox plugin. Proceed?", // Text
                              "Sandbox Plugin", // Title
                              MB_YESNO | MB_ICONQUESTION // Flags (yes/no buttons and question icon)
               ) == IDYES) {
                              // The user clicked "Yes", we may continue
                              return true;
               } else {
                              // The user clicked "No", so we need to abort
                              return false;
               }
}

bool onDoBattlerAction (RPG::Battler *battler, bool firstTry)
{
    if(firstTry)
    {
        MessageBox(
                    NULL, // We don't need a window handle
                    "Notice: battler action started.", // Text
                    "Sandbox Plugin", // Title
                    MB_OK // Flags
        );
    }
    return true;

}

bool onBattlerActionDone (RPG::Battler *battler, bool success)
{
    MessageBox(
                NULL, // We don't need a window handle
                "Notice: reached end of battler action.", // Text
                "Sandbox Plugin", // Title
                MB_OK // Flags
    );
    return true;
}

